package com.ho.test.xsl.validation.dtd;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import errorhandler.MyErrorHandler;

/**
 * Validate a XML with INTERNAL DTD and use of the standard DOM and SAX parser of Sun implementation
 * 
 * Validate a XML with EXTERNAL XSD and use of the standard DOM and SAX parser of Sun implementation
 * 
 * @author huseyin
 * 
 */
public class TestValidateXMLWithDTD {

	public static void main(String[] args) {
		try {
			{
				String xmlFileNameWithInternalDTD = "src/docs/dtd/people-internalDTD.xml";

				System.out.println("\n-------- validateXMLWithDTDAndDOM --------");
				System.out.println("---> Result of validation With Internal DTD: "+validateXMLWithDTDAndDOM(xmlFileNameWithInternalDTD));

				System.out.println("\n-------- validateXMLWithDTDAndSAX --------");
				System.out.println("---> Result of validation With Internal DTD: "+validateXMLWithDTDAndSAX(xmlFileNameWithInternalDTD));

			}
		
			{
				String xmlFileNameWithExternalDTD = "src/docs/dtd/people-externalDTD.xml";
				
				System.out.println("\n-------- validateXMLWithDTDAndDOM --------");
				System.out.println("---> Result of validation With External DTD: "+validateXMLWithDTDAndDOM(xmlFileNameWithExternalDTD));

				System.out.println("\n-------- validateXMLWithDTDAndSAX --------");
				System.out.println("---> Result of validation With External DTD: "+validateXMLWithDTDAndSAX(xmlFileNameWithExternalDTD));
				
			}
			
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	
	
	/**
	 * Validate a XML with DTD and use of the standard DOM parser of Sun implementation
	 * 
	 * @param xmlFileName
	 * @return
	 */
	public static boolean validateXMLWithDTDAndDOM(String xmlFileName) {
		try {
	         DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	         // Enable the document validation as the document is being parsed.
	         factory.setValidating(true);
	         factory.setNamespaceAware(true);

	         DocumentBuilder builder = factory.newDocumentBuilder();
	         builder.setErrorHandler(new MyErrorHandler());
	   
	         // Generates a Document object tree
	         Document xmlDocument = builder.parse(new InputSource(xmlFileName));
	         
//	         DOMSource source = new DOMSource(xmlDocument);
//	         StreamResult result = new StreamResult(System.out);
//	         //
//	         TransformerFactory transformerFactory = TransformerFactory.newInstance();
//	         Transformer transformer = transformerFactory.newTransformer();
//	         transformer.transform(source, result);
	         
	         return true;
	         
	      } catch (Throwable e) {
	         e.printStackTrace();
	         return false;
	      }
	}

	/**
	 * Validate a XML with DTD and use of the standard SAX parser of Sun implementation
	 * 
	 * @param xmlFileName
	 * @return
	 */
	public static boolean validateXMLWithDTDAndSAX(String xmlFileName) {
		try {
			SAXParserFactory factory = SAXParserFactory.newInstance();
			// Enable the document validation as the document is being parsed.
			factory.setValidating(true);
			factory.setNamespaceAware(true);

			SAXParser parser = factory.newSAXParser();

			XMLReader reader = parser.getXMLReader();
			reader.setErrorHandler(new MyErrorHandler());
			reader.parse(new InputSource(xmlFileName));

			return true;
	         
		} catch (Throwable e) {
			e.printStackTrace();
			return false;
		}
	}	

}
