package com.ho.test.xsl.validation.xsd;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import errorhandler.MyErrorHandler;

/**
 * Validate a XML with EXTERNAL XSD and use of the standard DOM and SAX parser of Sun implementation
 * 
 * @author huseyin
 * 
 */
public class TestValidateXMLWithExternalXSD {
	
	public static void main(String[] args) {
		try {
			String xmlFileName = "src/docs/people.xml";
			String xsdFileName = "src/docs/xsd/people.xsd";

			System.out.println("\n-------- validateXMLWithXSDAndDOM --------");
			System.out.println("---> Result of validation With External XSD: "+validateXMLWithXSDAndDOM(xmlFileName, xsdFileName));

			System.out.println("\n-------- validateXMLWithXSDAndSAX --------");
			System.out.println("---> Result of validation With External XSD: "+validateXMLWithXSDAndSAX(xmlFileName, xsdFileName));

			System.out.println("\n-------- validateXMLWithXSD --------");
			System.out.println("---> Result of validation With External XSD: "+validateXMLWithXSD(xmlFileName, xsdFileName));

		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	/**
	 * Validate a XML with EXTERNAL XSD and use of the standard DOM parser of Sun implementation
	 * 
	 * @param xmlFileName
	 * @param xsdFileName
	 * @return
	 */
	public static boolean validateXMLWithXSDAndDOM(String xmlFileName, String xsdFileName) {
        try {
        	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        	// Disable the document validation as the document is being parsed.
        	factory.setValidating(false);
        	factory.setNamespaceAware(true);
        	
        	SchemaFactory schemaFactory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
        	factory.setSchema(schemaFactory.newSchema(new StreamSource(new BufferedReader(new FileReader(xsdFileName)))));

        	DocumentBuilder builder = factory.newDocumentBuilder();
        	builder.setErrorHandler(new MyErrorHandler());

        	// Generates a Document object tree
        	Document document = builder.parse(new InputSource(xmlFileName));

            return true;
        }catch (Throwable ex) {
        	ex.printStackTrace();
            return false;
        }  
    }
	
	/**
	 * Validate a XML with EXTERNAL XSD and use of the standard SAX parser of Sun implementation
	 * 
	 * @param xmlFileName
	 * @param xsdFileName
	 * @return
	 */
	public static boolean validateXMLWithXSDAndSAX(String xmlFileName, String xsdFileName) {
		try {
			SAXParserFactory factory = SAXParserFactory.newInstance();
			// Disable the document validation as the document is being parsed.
			factory.setValidating(false);
			factory.setNamespaceAware(true);

			SchemaFactory schemaFactory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
			factory.setSchema(schemaFactory.newSchema(new StreamSource(new BufferedReader(new FileReader(xsdFileName)))));
			
			SAXParser parser = factory.newSAXParser();
			
			XMLReader reader = parser.getXMLReader();
			reader.setErrorHandler(new MyErrorHandler());
			reader.parse(new InputSource(xmlFileName));

			return true;
	         
		} catch (Throwable e) {
			e.printStackTrace();
			return false;
		}
	}	
	
	
	
	/**
	 * Validate a XML with XSD validator which compiles the XSD schema and applies it on XML stream.
	 * 
	 * @param xmlFileName
	 * @param xsdFileName
	 * @return
	 */
	public static boolean validateXMLWithXSD(String xmlFileName, String xsdFileName) {
        try {
	        // Lookup a factory for the W3C XML Schema language
	        SchemaFactory factory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");

	        // Compile the schema.
	        Schema schema = factory.newSchema(new StreamSource(new BufferedReader(new FileReader(xsdFileName))));
	        
	        // Get a validator from the schema.
	        Validator validator = schema.newValidator();
	        
	        // Parse the document
	        String xml = getContentFromFilename(xmlFileName);
	        Source source = new StreamSource(new StringReader(xml));
	        
	        // Check the document
            validator.validate(source);
        	
            return true;
        }catch (Throwable ex) {
        	ex.printStackTrace();
            return false;
        }finally{
        }  
    }
	
	/**
	 * Get the content of a file
	 * @param filename
	 * @return
	 */
	public static String getContentFromFilename(String filename){
		BufferedReader br = null;
		String everything = null;
	    try { 
	    	br = new BufferedReader(new FileReader(filename)); 
	        StringBuilder sb = new StringBuilder(); 
	        String line = br.readLine(); 
	 
	        while (line != null) { 
	            sb.append(line); 
	            sb.append("\n"); 
	            line = br.readLine(); 
	        } 
	        everything = sb.toString(); 
	        
	    }catch (Throwable ex) {
        	ex.printStackTrace();
        } finally { 
	        try {
				br.close();
			} catch (IOException e) {
				e.printStackTrace();
			} 
	    } 
	    return everything;
	}


}
