package com.ho.test.xsl.validation.xsd;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import errorhandler.MyErrorHandler;

/**
 * Validate a XML with INTERNAL XSD and use of the standard DOM and SAX parser of Sun implementation
 * 
 * @author huseyin
 * 
 */
public class TestValidateXMLWithInternalXSD {

	public static void main(String[] args) {
		try {
			String xmlFileNameWithXSD = "src/docs/xsd/people-XSD.xml";
			String xsdFileName = "src/docs/xsd/people.xsd";

			System.out.println("\n-------- validateXMLWithXSDAndDOM --------");
			System.out.println("---> Result of validation With Internal XSD: "+validateXMLWithXSDAndDOM(xmlFileNameWithXSD));

			System.out.println("\n-------- validateXMLWithXSDAndSAX --------");
			System.out.println("---> Result of validation With Internal XSD: "+validateXMLWithXSDAndSAX(xmlFileNameWithXSD));

		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	/**
	 * Validate a XML with INTERNAL XSD and use of the standard DOM parser of Sun implementation
	 * 
	 * @param xmlFileName
	 * @return
	 */
	public static boolean validateXMLWithXSDAndDOM(String xmlFileName) {
        try {
        	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        	// Enable the document validation as the document is being parsed.
        	factory.setValidating(true);
        	factory.setNamespaceAware(true);
        	factory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaLanguage", "http://www.w3.org/2001/XMLSchema");

        	DocumentBuilder builder = factory.newDocumentBuilder();
        	builder.setErrorHandler(new MyErrorHandler());

        	// Generates a Document object tree
        	Document document = builder.parse(new InputSource(xmlFileName));

            return true;
        }catch (Throwable ex) {
        	ex.printStackTrace();
            return false;
        }  
    }
	
	/**
	 * Validate a XML with INTERNAL XSD and use of the standard SAX parser of Sun implementation
	 * 
	 * @param xmlFileName
	 * @return
	 */
	public static boolean validateXMLWithXSDAndSAX(String xmlFileName) {
		try {
			SAXParserFactory factory = SAXParserFactory.newInstance();
			// Enable the document validation as the document is being parsed.
			factory.setValidating(true);
			factory.setNamespaceAware(true);

			SAXParser parser = factory.newSAXParser();
			parser.setProperty("http://java.sun.com/xml/jaxp/properties/schemaLanguage", "http://www.w3.org/2001/XMLSchema");
			
			XMLReader reader = parser.getXMLReader();
			reader.setErrorHandler(new MyErrorHandler());
			reader.parse(new InputSource(xmlFileName));

			return true;
	         
		} catch (Throwable e) {
			e.printStackTrace();
			return false;
		}
	}	

}
