package com.ho.test.xsl.welformed;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import errorhandler.MyErrorHandler;

/**
 * Checking Well formedness of a XML with use of the standard DOM and SAX parser of Sun implementation
 * 
 * @author huseyin
 * 
 */
public class TestWellFormednessXML {

	public static void main(String[] args) {
		try {
			String xmlFileName = "src/docs/people.xml";

			System.out.println("\n-------- checkWellFormednessXMLWithDOM --------");
			System.out.println("---> Result of checking Well Formedness: " + checkWellFormednessXMLWithDOM(xmlFileName));

			System.out.println("\n-------- checkWellFormednessXMLWithSAX --------");
			System.out.println("---> Result of checking Well Formedness: " + checkWellFormednessXMLWithSAX(xmlFileName));

		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	/**
	 * Checking Well formedness of a XML with use of the standard DOM parser of Sun implementation
	 * 
	 * @param xmlFileName
	 * @return
	 */
	public static boolean checkWellFormednessXMLWithDOM(String xmlFileName) {
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			
			// Disable the document validation as the document is being parsed.
			factory.setValidating(false);
			factory.setNamespaceAware(true);

			DocumentBuilder builder = factory.newDocumentBuilder();
			builder.setErrorHandler(new MyErrorHandler());

			// Generates a Document object tree
			Document xmlDocument = builder.parse(new InputSource(xmlFileName));

			return true;

		} catch (Throwable e) {
			e.printStackTrace();
			return false;
		}
	}

	/**
	 * Checking Well formedness of a XML with use of the standard SAX parser of Sun implementation
	 * 
	 * @param xmlFileName
	 * @return
	 */
	public static boolean checkWellFormednessXMLWithSAX(String xmlFileName) {
		try {
			SAXParserFactory factory = SAXParserFactory.newInstance();
			
			// Disable the document validation as the document is being parsed.
			factory.setValidating(false);
			factory.setNamespaceAware(true);

			SAXParser parser = factory.newSAXParser();

			XMLReader reader = parser.getXMLReader();
			reader.setErrorHandler(new MyErrorHandler());

			// Parse the XML streamm
			reader.parse(new InputSource(xmlFileName));

			return true;

		} catch (Throwable e) {
			e.printStackTrace();
			return false;
		}
	}
}
